use std::error::Error;
use mailparse::ParsedMail;

// less efficient, easier api
pub fn xml_safe(text: &str) -> String {
    // note we escape more than we need to
    text.chars()
        .map(|c| {
            match c {
                '&' => "&amp;",
                '<' => "&lt;",
                '>' => "&gt;",
                '"' => "&quot;",
                '\'' => "&#39;",
                // Quick and dirty email obfuscation
                '@' => "&#x40;",
                // need an explicit return here because c is not a &str
                _ => return c.to_string(),
            }
            .to_string()
        })
        .collect::<String>()
}

pub fn extract_html_content(part: &ParsedMail) -> Option<String> {
    if part.ctype.mimetype == "text/html" {
        return part.get_body().ok();
    }
    for subpart in &part.subparts {
        if let Some(html) = extract_html_content(subpart) {
            return Some(html);
        }
    }
    None
}

pub fn parse_email(parsed_mail: &ParsedMail) -> Result<String, Box<dyn Error>> {
    if let Some(html_content) = extract_html_content(&parsed_mail) {
        return Ok(html_content.replace("\r\n", ""));
    } else if parsed_mail.ctype.mimetype.starts_with("text/plain") {
        // Convert plain text to simple HTML
        let plain_text = parsed_mail.get_body()?.replace("\r\n", "<br>");
        let html_content = format!("<html><body>{}</body></html>", plain_text);
        return Ok(html_content);
    }

    Err(From::from("No HTML or text content found"))
}
